#!/usr/bin/python
# encoding=utf8
import base64
import os


class Img2Base64(object):
    '''
    将图片变成base64编码，然后在web中打开，不用引入图片外部图片文件
    '''

    def __init__(self, sImgDir):
        fr = open(sImgDir, 'rb')
        self.hexImg = base64.b64encode(fr.read())
        fr.close()


'''if __name__=='__main__':

    print ' <img src = "data:image/bmp;base64,'+Img2Base64('/Users/chancriss/Desktop/WorkSpace/PythonSpace/EmmaTools/ocr.jpg').hexImg+'\"/>'
'''



# encoding=utf8
# !/usr/bin/python

import os
import subprocess
from Img2Base64 import Img2Base64
import time
import shutil


class alasRun(object):
    '''
    多场景的混合运行类，可以执行在test workspace下的全部测试场景，
    支持并发和线性执行两种模式
    this is a class for merge all performance testing scenario
     that scenario under the current test workspace dir
    '''

    def __init__(self):

        self.strCurPath = os.path.abspath(os.curdir)  # get the current work space

        self.scenariolist = []
        # self._collectReport()

        # self.startTime=time.strftime("%Y-%m-%d %H:%M:%S",time.localtime(time.time()))
        # self.endTime=time.strftime("%Y-%m-%d %H:%M:%S",time.localtime(time.time()))

    def _collectScenario(self):
        '''
        collect performance test scenario that under test workspace name
        Returns:
        null
        '''
        for root, dirs, files in os.walk(self.strCurPath):
            # print dirs

            for file in files:

                if file.find('script.py') >= 0 and root.find('Parralle_Scenario') < 0 and file[len(file) - 1] != 'c':
                    tempscenario = root[len(self.strCurPath):]
                    tempscenario = tempscenario[1:tempscenario.rfind('/')]
                    # print tempscenario
                    self.scenariolist.append(tempscenario)

    def _SerialReportIndex(self, ScenarioList, sResult):
        '''
        make the serial report index page
        Args:
            ScenarioList: scenario list
            sResult: the result dir

        Returns:

        '''
        wf = open(sResult + 'index.html', 'w')
        wf.write('''<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN"
    "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">
<html xmlns="http://www.w3.org/1999/xhtml" xml:lang="en" lang="en">
<head>
    <title>Serial Report</title>
    <meta http-equiv="Content-Type" content="text/html; charset=iso-8859-1" />
    <meta http-equiv="Content-Language" content="en" />
    <style type="text/css">
        body {
            background-color: #FFFFFF;
            color: #000000;
            font-family: Verdana, sans-serif;
            font-size: 11px;
            padding: 5px;
        }
        h1 {
            font-size: 16px;
            background: #FF9933;
            margin-bottom: 0;
            padding-left: 5px;
            padding-top: 2px;
        }
        h2 {
            font-size: 13px;
            background: #C0C0C0;
            padding-left: 5px;
            margin-top: 2em;
            margin-bottom: .75em;
        }
        h3 {
            font-size: 12px;
            background: #EEEEEE;
            padding-left: 5px;
            margin-bottom: 0.5em;
        }
        h4 {
            font-size: 11px;
            padding-left: 20px;
            margin-bottom: 0;
        }
        p {
            margin: 0;
            padding: 0;
        }
        table {
            margin-left: 10px;
        }
        td {
            text-align: right;
            color: #000000;
            background: #FFFFFF;
            padding-left: 10px;
            padding-right: 10px;
            padding-bottom: 0;
        }
        th {
            text-align: center;
            padding-right: 10px;
            padding-left: 10px;
            color: #000000;
            background: #FFFFFF;
        }
        div.summary {
            padding-left: 20px;
        }
    </style>
</head>
<body>

<h1>Performance Results Report List</h1>

<h2>Summary</h2><div class="summary">''')
        wf.write('<b>test start:</b> ' + self.startTime + '<br /><br />')
        wf.write('<b>test finish:</b> ' + self.endTime + '<br /><br />')
        wf.write('''<h2>All Scenario </h2>
<h3>Detail</h3>
<table>
<tr><th>Scanrio Name </th><th>Report link</th></tr>
''')
        for Scenario in ScenarioList:
            wf.write(
                '<tr><td>' + Scenario + '</td><td><a href=\'' + Scenario + '.html\' target="_blank">' + Scenario + '</a></td></tr>')
        wf.write('''</table>
</body>
</html>''')
        wf.close()

    def _ImgtoHTMLBase(self, imgfile):
        sImgBase64 = ' <img src = "data:image/bmp;base64,' + Img2Base64(
            os.path.join(imgfile)).hexImg + '\"/>'
        return sImgBase64

    def _collectReport(self):
        '''
        this is a collection all scenrio report fun,and output the serial_result dir
        Returns:
        the report dir
        '''
        if not os.path.exists(self.strCurPath + '/Report/Serial_Result/'):
            os.makedirs(self.strCurPath + '/Report/Serial_Result/')
        sCurResult = self.strCurPath + '/Report/Serial_Result/Serial_Result' + time.strftime("%Y%m%d%H%M%S",
                                                                                             time.localtime(
                                                                                                 time.time())) + '/'

        os.makedirs(sCurResult)

        aTempReportList = []
        # aTempReportfilelist=[]
        for scenario in self.scenariolist:
            aTempstr = self.strCurPath + '/' + scenario + '/results/'
            for root, dirs, files in os.walk(aTempstr):
                for file in files:
                    if file == 'results.html':
                        # All_Transactions_response_times_intervals.png
                        # All_Transactions_response_times.png
                        # All_Transactions_throughput.png
                        reportfile = os.path.join(root, file)
                        rf = open(reportfile, 'r')
                        sResult = rf.read()
                        rf.close()
                        sbtimesintervals = self._ImgtoHTMLBase(
                            os.path.join(root, 'All_Transactions_response_times_intervals.png'))

                        sbtimes = self._ImgtoHTMLBase(os.path.join(root, 'All_Transactions_response_times.png'))
                        sthroughput = self._ImgtoHTMLBase(os.path.join(root, 'All_Transactions_throughput.png'))
                        sResult = sResult.replace('<img src="All_Transactions_response_times_intervals.png"></img>',
                                                  sbtimesintervals)
                        sResult = sResult.replace('<img src="All_Transactions_response_times.png"></img>',
                                                  sbtimes)
                        sResult = sResult.replace('<img src="All_Transactions_throughput.png"></img>',
                                                  sthroughput)
                        aTempReportList.append(scenario)

                        sTempResultfile = sCurResult + scenario + '.html'

                        wf = open(sTempResultfile, 'w')
                        wf.write(sResult)
                        wf.close()

        self._SerialReportIndex(aTempReportList, sCurResult)

    def _mvReport(self):

        sCurReport = self.strCurPath + '/Report/Parralle_Result/'

        sOldReport = self.strCurPath + '/Parralle_Scenario/results/'

        if os.path.exists(sCurReport):
            shutil.rmtree(sCurReport)
        shutil.copytree(sOldReport, sCurReport)

    def _clearHistoryScenrioResult(self):
        '''
        clear all scenrio history result
        Returns:

        '''
        for scenario in self.scenariolist:
            aTempstr = self.strCurPath + '/' + scenario + '/results/'
            if os.path.exists(aTempstr):
                shutil.rmtree(aTempstr)

    def sRun(self):
        '''
        serial process all scenario under test work space
        Returns:

        '''
        self.scenariolist = []
        self._collectScenario()
        if len(self.scenariolist) <= 0:
            print 'No Performance Test Scenario Under Current Test WorkSpace'
            return 0
        self._clearHistoryScenrioResult()

        i = 0
        self.startTime = time.strftime("%Y-%m-%d %H:%M:%S", time.localtime(time.time()))
        print 'Serial Run:'
        # print self.scenariolist
        for aline in self.scenariolist:
            i = i + 1
            print '*****************************************************************'
            print 'Run Scenario ' + str(i) + '   :  ' + aline
            print '*****************************************************************'
            sTempCommand = 'multimech-run ' + aline

            sSubp = subprocess.Popen(sTempCommand, shell=True, stdout=subprocess.PIPE)
            while sSubp.poll() == None:
                print sSubp.stdout.readline()
        self.endTime = time.strftime("%Y-%m-%d %H:%M:%S", time.localtime(time.time()))
        self._collectReport()

    def _ParrallelConfig(self, sConfig):
        wf = open(sConfig, 'w')
        wf.write('''[global]
run_time = 30
rampup = 0
results_ts_interval = 1
progress_bar = on
console_logging = on
xml_report = off\r\n''')
        i = 1
        # print self.scenariolist
        for scenariotemp in self.scenariolist:
            wf.write('[user_group - ' + str(i) + ']\r\n')
            i = i + 1
            wf.write('threads = 1\r\n')
            wf.write('script = ' + scenariotemp + 'v_user.py\r\n')
        wf.close()

    def _MergParralleScript(self):

        sCurScenario = self.strCurPath + '/Parralle_Scenario/'
        if not os.path.exists(sCurScenario):
            os.makedirs(sCurScenario)
        self.scenariolist = []
        self._collectScenario()
        if len(self.scenariolist) <= 0:
            # print 'No Performance Test Scenario Under Current Test WorkSpace'
            return 0
        sCruScenarioTestScript = sCurScenario + 'test_scripts/'
        if os.path.exists(sCruScenarioTestScript):
            shutil.rmtree(sCruScenarioTestScript)
        os.makedirs(sCruScenarioTestScript)
        sConfig = sCurScenario + 'config.cfg'
        self._ParrallelConfig(sConfig)
        for ascenario in self.scenariolist:

            for root, dirs, files in os.walk(self.strCurPath + '/' + ascenario + '/test_scripts/'):
                for file in files:
                    shutil.copyfile(os.path.join(self.strCurPath + '/' + ascenario + '/test_scripts/', file),
                                    sCruScenarioTestScript + file)
                    # shutil.copyfile

    def pRun(self):
        self._MergParralleScript()
        if len(self.scenariolist) <= 0:
            print 'No Performance Test Scenario Under Current Test WorkSpace'
            return 0
        # self.startTime = time.strftime("%Y-%m-%d %H:%M:%S", time.localtime(time.time()))
        sTempCommand = 'multimech-run Parralle_Scenario'
        sSubp = subprocess.Popen(sTempCommand, shell=True, stdout=subprocess.PIPE)
        while sSubp.poll() == None:
            print sSubp.stdout.readline()

        # self.endTime = time.strftime("%Y-%m-%d %H:%M:%S", time.localtime(time.time()))
        self._mvReport()


'''
if __name__ == '__main__':
    arun = alasRun()
    arun.pRun()
    arun.sRun()
'''



#!/usr/bin/env python
# coding=utf-8
'''
author:Crisschan
time:2016-6-30
'''
from mitmproxy import controller, proxy
import os
import datetime
from F2requests import F2requests
from Cfg import Cfg


class FriedRing(controller.Master):
    # fscript =
    def __init__(self, server, fnamescript):
        curscenrioscript = fnamescript
        curpath = os.path.abspath(os.curdir)
        fscriptsolutionpath = os.path.join(curpath, fnamescript)
        if not os.path.isdir(fscriptsolutionpath):
            os.makedirs(fscriptsolutionpath)
        else:
            fnamescript = fnamescript + str(datetime.datetime.now().microsecond)
            fscriptsolutionpath = os.path.join(curpath, fnamescript)
            os.makedirs(fscriptsolutionpath)

        Cfg(fscriptsolutionpath, curscenrioscript)
        self.fnamescript = str(fscriptsolutionpath) + '/test_scripts/' + curscenrioscript + 'script.py'
        print 'script solution path(include script files, config files and results:' + str(fscriptsolutionpath)

        controller.Master.__init__(self, server)
        self.f2r = F2requests(self.fnamescript)

    # def shutdown(self):
    #   self.shutdown()
    def run(self):
        try:
            return controller.Master.run(self)
        except KeyboardInterrupt:
            self.shutdown()

    def handle_request(self, msg):
        # print msg
        req = msg.request
        print str(req.host) + str(req.path)
        self.f2r.F2Req(req)
        msg.reply()

    def handle_response(self, msgg):
        # print msg
        msgg.reply()
        res = msgg.response
        '''
        print res.status_code
        print res.headers
        print res.content+'\n'
        print res.reason+'\n'
        print res.timestamp_start+'\n'
        print res.timestamp_end+'\n'
        print '--------------------------------------\n'
        '''


'''
if  __name__ == '__main__':
    opts = options.Options(cadir="~/.mitmproxy/", listen_port=8888)
    config = proxy.ProxyConfig(opts)
    server = proxy.ProxyServer(config)
    m = FriedRing(server,'/Users/chancriss/Desktop/WorkSpace/PythonSpace/FriedRingWorkSpace/')
    m.run()
'''



import os


class Cfg():
    def __init__(self, fscriptsolutionpath, curscenrioscript):
        self.fscriptsolutionpath = fscriptsolutionpath
        self.curscenrioscript = curscenrioscript
        self.__Setcfg()
        self.__SetVU()

    def __Setcfg(self):
        fcfgpath = str(self.fscriptsolutionpath) + '/config.cfg'
        fcfg = open(fcfgpath, 'w')
        scfglines = '[global]\r\nrun_time = 30\r\nrampup = 0\r\nresults_ts_interval = 1\r\nprogress_bar = on\r\nconsole_logging = on\r\nxml_report = off\r\n\r\n[user_group - 1]\r\nthreads = 1\r\nscript = ' + self.curscenrioscript + 'v_user.py'
        fcfg.write(scfglines)
        fcfg.close()

    def __SetVU(self):
        fV_UserPath = os.path.join(self.fscriptsolutionpath, 'test_scripts')
        if not os.path.isdir(fV_UserPath):
            os.makedirs(fV_UserPath)
        fV_Userfile = str(fV_UserPath) + '/' + self.curscenrioscript + 'v_user.py'
        fV_Userscript = open(fV_Userfile, 'w')
        sScript = '#!/usr/bin/env python\r\n#coding=utf-8\r\nimport requests\r\nimport json\r\nimport time\r\nimport ' + self.curscenrioscript + 'script\r\nclass Transaction(object):\r\n    def __init__(self):\r\n        #self.custom_timers={}\r\n        pass\r\n    def run(self):\r\n        #start_timer = time.time()\r\n        ' + self.curscenrioscript + 'script.script()\r\n        #latency = time.time() - start_timer\r\n        #user\'s transaction \r\n        #self.custom_timers[\'Transaction_Custom\']=latency\r\nif __name__ == \'__main__\':\r\n    trans = Transaction()\r\n    trans.run()\r\n    #print trans.custom_timers'
        fV_Userscript.write(sScript)
        fV_Userscript.close()



#!/usr/bin/env python
# coding=utf-8
import getopt
import os
import time
import sys
from mitmproxy import proxy
from FriedRing import FriedRing
from alasRun import alasRun

VERSION = '2.0.0'


def main():
    opts, args = getopt.getopt(sys.argv[1:], "hpr:w:")
    strPort = 8888
    fnamescript = '__crisschan_TEMP' + str(time.time())
    for op, value in opts:
        if op == "-p":
            strPort = value
        elif op == "-w":
            fnamescript = value
        elif op == "-r":
            sTargetRun = value
        elif op == "-h":
            # usage()
            print '-p the proxy port\r\n-r the run all scenario under test workspace,the param is:\r\n    s - is serial run all scenrio \r\n    p - is parralle run all scenrio\r\n-w the performance test workspace name \r\n'
            sys.exit()

    config = proxy.ProxyConfig(
        cadir=os.path.expanduser("~/.mitmproxy/"),
        port=int(strPort)
    )

    if op == '-r':
        ARun = alasRun()
        if sTargetRun == 's':
            ARun.sRun()
        elif sTargetRun == 'p':
            ARun.pRun()
        else:
            print '-r is the error input!!!!'
            return 0
    else:
        server = proxy.ProxyServer(config)
        print 'the porxy port is ' + str(strPort)
        m = FriedRing(server, fnamescript)
        m.run()


if __name__ == "__main__":
    main()



#!/usr/bin/env python
#coding=utf-8
'''
author:Crisschan
time:2016-7-1

author:Crisschan
time 2016-7-11 modi for the headers check the " to \"
'''
from mitmproxy.models.http import HTTPRequest
from mitmproxy.models.http import HTTPResponse
class F2requests():
    def __init__(self,fnamescript):
        self.fscript = open(fnamescript,'w+')
        self.__InitScript()
    def __del__(self):
        self.fscript.close()
    def F2Req(self,request):
        self.request = request
        if str(self.request.method) == 'GET':
            self.__F2Get()
        else:
            self.__F2Post()

    def __InsertHost(self):
        # insert the request host information
        self.fscript.write(self.__Insert8blank()+'######'+str(self.request.host)+'#######')
        # insert the request host information
    def __InsertTimeStampStart(self):
        # insert the request host information
        self.__InsertBlankRow()
        self.__InsertHost()
        self.__InsertBlankRow()
        self.fscript.write(self.__Insert8blank()+'# start : ')
        self.fscript.write(str(self.request.timestamp_start))

        self.__InsertBlankRow()

    def __InsertTimeStampEnd(self):
        # inser the time stamp of end
        self.__InsertBlankRow()
        self.fscript.write(self.__Insert8blank()+'# end : ')
        self.fscript.write(str(self.request.timestamp_end))

        self.__InsertBlankRow()

    def __InitScript(self):
        # insert the global import
        strPreScript = '#!/usr/bin/env python\r\n'
        strPreScript=strPreScript+'#coding=utf-8\r\n'
        strPreScript=strPreScript+'import requests\r\nimport json\r\nimport time\r\n\r\ndef script():'
        self.fscript.write(strPreScript)
        self.__InsertBlankRow()

    def __InsertBlankRow(self):
        # insert a blank row to script
        self.fscript.write('\r\n')

    def __InserAssert(self):
        # insert the assert target point
        self.__InsertBlankRow()
        self.fscript.write(self.__Insert8blank()+'#insert the assert or other check point    #exp. print r.text')
        self.__InsertBlankRow()

    def __F2Header(self):
        # insert the http header
        # modi add the replace " to \"
        strT = str(self.request.headers)
        strT= strT.replace('\"','\\\"')
        strT = '\"' + strT
        strT = strT.replace('\r\n', '\n')
        strT = strT.replace('\n', '","')
        #strT = strT.replace('\n', '",\r\n"') #for format header
        strT = strT.replace(': ', '":"')
        strT = '{' + strT[:-2] + '}'
        return strT
    def __Insert8blank(self):
        return '      '
    def __F2Get(self):
        #print 'get --'
        self.__InsertTimeStampStart()
        #insert the header
        self.fscript.write(self.__Insert8blank()+'headers='+self.__F2Header())
        self.__InsertBlankRow()
        # insert the get request
        strRquest = 'http://'+str(self.request.host)+str(self.request.path)
        strRquest=self.__Insert8blank()+'r=requests.get("'+strRquest+'",headers=headers)'
        self.fscript.write(strRquest)
        self.__InsertTimeStampEnd()
        self.__InserAssert()
    def __F2Post(self):
        self.__InsertTimeStampStart()
        self.__InsertBlankRow()
        self.fscript.write(self.__Insert8blank()+'headers=' + self.__F2Header())
        self.__InsertBlankRow()
        self.fscript.write(self.__Insert8blank()+'payload = "'+self.request.content+'"')
        self.__InsertBlankRow()
        strRquest = 'http://'+str(self.request.host) + str(self.request.path)
        strRquest = self.__Insert8blank()+'r=requests.post("' + strRquest + '",data=payload,headers=headers)'
        self.fscript.write(strRquest)
        self.__InsertTimeStampEnd()
        self.__InserAssert()




#!/usr/bin/env python
#coding=utf-8
import codecs
import os
import sys
try:
    from setuptools import setup,find_packages
except:
    from distutils.core import setup
'''import the setup lib'''


'''
    定义一个read方法，用来读取目录下的长描述
    我们一般是将README文件中的内容读取出来作为长描述，这个会在PyPI中你这个包的页面上展现出来，
    你也可以不用这个方法，自己手动写内容即可，
    PyPI上支持.rst格式的文件。暂不支持.md格式的文件，<BR>.rst文件PyPI会自动把它转为HTML形式显示在你包的信息页面上。
'''
def read(fname):
    return codecs.open(os.path.join(os.path.dirname(__file__), fname)).read()
NAME = "FriedRing"
'''
名字，一般放你包的名字即可
'''
PACKAGES = find_packages(include=[
        "FriedRing", "FriedRing.*"
    ])
'''
包含的包，可以多个，这是一个列表
'''
DESCRIPTION = "this is a FriedRing package for get http request and response."
'''
关于这个包的描述
'''
LONG_DESCRIPTION = read("README.md")
'''
参见read方法说明
'''
KEYWORDS = "FriedRing python package"
'''
关于当前包的一些关键字，方便PyPI进行分类。
'''
AUTHOR = "CrissChan"

AUTHOR_EMAIL = "can101208@gmail.com"
'''
作者的邮件地址
'''
URL = "http://blog.csdn.net/crisschan"
'''
你这个包的项目地址，如果有，给一个吧，没有你直接填写在PyPI你这个包的地址也是可以的
'''
VERSION = "2.0.0"
'''
当前包的版本，这个按你自己需要的版本控制方式来
'''
LICENSE = "MIT"
'''
授权方式，我喜欢的是MIT的方式，你可以换成其他方式
'''
REQUIREMENTS = [i.strip() for i in open("requirement.txt").readlines()]

'''
this grabs the requirements from requirements.txt
'''
setup(
    name = NAME,
    version = VERSION,
    description = DESCRIPTION,
    long_description = LONG_DESCRIPTION,
    install_requires=REQUIREMENTS,
    classifiers=[
        "License :: OSI Approved :: MIT License",
        "Development Status :: 5 - Production/Stable",
        "Environment :: Console",
        "Environment :: Console :: Curses",
        "Operating System :: MacOS :: MacOS X",
        "Operating System :: POSIX",
        "Programming Language :: Python",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: Implementation :: CPython",
        "Programming Language :: Python :: Implementation :: PyPy",
        "Topic :: Internet",
        "Topic :: Internet :: WWW/HTTP",
        "Topic :: Software Development :: Testing"
    ],

    keywords = KEYWORDS,
    author = AUTHOR,
    author_email = AUTHOR_EMAIL,
    url = URL,
    license = LICENSE,
    packages = PACKAGES,
    include_package_data=True,
    zip_safe=True,
    entry_points={
        'console_scripts': [
            "fr = FriedRing.fr:main"
        ]
    },
    # scripts=["scripts/test.py"],scripts表示将该文件放到 Python的Scripts目录下，可以直接用
)
## 把上面的变量填入了一个setup()中即可。